"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.create = void 0;
const utils_1 = require("../../../utils");
const CallRequest = __importStar(require("./CallRequest"));
const HeaderParameter = __importStar(require("./HeaderParameter"));
const PathParameter = __importStar(require("./PathParameter"));
const QueryParameter = __importStar(require("./QueryParameter"));
const createEncodingMap_1 = require("./createEncodingMap");
const create = (factory, params, methodType) => {
    var _a;
    const statements = [];
    const { convertedParams, operationParams } = params;
    const { pickedParameters } = convertedParams;
    // Generate Path Parameter
    const pathParameters = pickedParameters.filter(PathParameter.isPathParameter);
    statements.push(PathParameter.create(factory, params.operationParams.requestUri, pathParameters, methodType));
    /**
     * Create Variable: const header = {};
     */
    const initialHeaderObject = {};
    if (convertedParams.has2OrMoreRequestContentTypes) {
        initialHeaderObject["Content-Type"] = {
            type: "variable",
            value: `params.headers.Content-Type`,
        };
    }
    else if (convertedParams.requestFirstContentType) {
        initialHeaderObject["Content-Type"] = {
            type: "string",
            value: convertedParams.requestFirstContentType,
        };
    }
    if (convertedParams.has2OrMoreSuccessResponseContentTypes) {
        initialHeaderObject["Accept"] = {
            type: "variable",
            value: `params.headers.Accept`,
        };
    }
    else if (convertedParams.successResponseFirstContentType) {
        initialHeaderObject["Accept"] = {
            type: "string",
            value: convertedParams.successResponseFirstContentType,
        };
    }
    // Generate Header Parameter
    const headerParameter = pickedParameters.filter(item => item.in === "header");
    const headerObject = Object.values(headerParameter).reduce((previous, current) => {
        return { ...previous, [current.name]: { type: "variable", value: `params.parameter.${current.name}` } };
    }, initialHeaderObject);
    statements.push(HeaderParameter.create(factory, {
        variableName: "headers",
        object: headerObject,
    }));
    /**
     * Create Variable: const requestEncoding = {};
     */
    const content = (_a = operationParams.requestBody) === null || _a === void 0 ? void 0 : _a.content;
    if (content) {
        const encodingMap = (0, createEncodingMap_1.createEncodingMap)(content);
        let identifier;
        if (convertedParams.has2OrMoreRequestContentTypes) {
            identifier = factory.Identifier.create({
                name: JSON.stringify(encodingMap, null, 2),
            });
        }
        else if (convertedParams.requestFirstContentType) {
            identifier = factory.Identifier.create({
                name: JSON.stringify({ [convertedParams.requestFirstContentType]: encodingMap[convertedParams.requestFirstContentType] }, null, 2),
            });
        }
        const requestEncodingsVariableStatement = factory.VariableStatement.create({
            declarationList: factory.VariableDeclarationList.create({
                flag: "const",
                declarations: [
                    factory.VariableDeclaration.create({
                        name: "requestEncodings",
                        initializer: identifier,
                        type: factory.TypeReferenceNode.create({
                            name: "Record<string, Record<string, Encoding>>",
                        }),
                    }),
                ],
            }),
        });
        if (identifier && Object.keys(encodingMap).length > 0) {
            statements.push(requestEncodingsVariableStatement);
        }
    }
    // Generate Query Parameter
    if (convertedParams.hasQueryParameters) {
        const queryParameter = pickedParameters.filter(item => item.in === "query");
        const queryObject = Object.values(queryParameter).reduce((previous, current) => {
            const { text, escaped } = (0, utils_1.escapeText2)(current.name);
            const variableDeclareText = escaped ? `params.parameter[${text}]` : `params.parameter.${text}`;
            return {
                ...previous,
                [current.name]: { type: "variable", value: variableDeclareText, style: current.style, explode: !!current.explode },
            };
        }, {});
        statements.push(QueryParameter.create(factory, { variableName: "queryParameters", object: queryObject }));
    }
    // Generate CallRequest
    statements.push(factory.ReturnStatement.create({
        expression: CallRequest.create(factory, params, methodType),
    }));
    return statements.length === 0 ? [factory.ReturnStatement.create({})] : statements;
};
exports.create = create;
//# sourceMappingURL=index.js.map