"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.make = exports.create = void 0;
const typescript_1 = __importDefault(require("typescript"));
const utils_1 = require("./utils");
const create = ({ factory }) => (params) => {
    const createNode = () => {
        if (typeof params.value === "string") {
            const literal = typescript_1.default.setEmitFlags(factory.createStringLiteral(params.value), typescript_1.default.EmitFlags.NoAsciiEscaping);
            return factory.createLiteralTypeNode(literal);
        }
        if (typeof params.value === "number") {
            return factory.createLiteralTypeNode(factory.createNumericLiteral(params.value));
        }
        return factory.createLiteralTypeNode(params.value ? factory.createTrue() : factory.createFalse());
    };
    const node = createNode();
    if (params.comment) {
        const comment = (0, utils_1.generateComment)(params.comment);
        return typescript_1.default.addSyntheticLeadingComment(node, typescript_1.default.SyntaxKind.MultiLineCommentTrivia, comment.value, comment.hasTrailingNewLine);
    }
    return node;
};
exports.create = create;
const make = (context) => {
    return {
        create: (0, exports.create)(context),
    };
};
exports.make = make;
//# sourceMappingURL=LiteralTypeNode.js.map