import * as Utils from "../utils";
import { createEncodingMap } from "./createEncodingMap";
/**
 *
 * const encodingMap = {
 *   "application/json": {},
 *   "application/x-www-form-urlencoded": {},
 * }
 */
const createEncodingParams = (factory, params) => {
    var _a;
    const content = (_a = params.operationParams.requestBody) === null || _a === void 0 ? void 0 : _a.content;
    if (!content) {
        return;
    }
    const encodingMap = createEncodingMap(content);
    if (Object.keys(encodingMap).length === 0) {
        return;
    }
    if (params.convertedParams.has2OrMoreRequestContentTypes) {
        return factory.Identifier.create({ name: `requestEncodings[params.headers["Content-Type"]]` });
    }
    return factory.Identifier.create({ name: `requestEncodings["${params.convertedParams.requestFirstContentType}"]` });
};
/**
 * this.apiClient.request("GET", url, requestBody, headers, queryParameters);
 */
export const create = (factory, params, methodType) => {
    const { convertedParams } = params;
    const apiClientVariableIdentifier = {
        class: "this.apiClient.request",
        function: "apiClient.request",
        "currying-function": "apiClient.request",
    };
    const expression = Utils.generateVariableIdentifier(factory, apiClientVariableIdentifier[methodType]);
    const requestBodyEncoding = createEncodingParams(factory, params);
    const requestArgs = factory.ObjectLiteralExpression.create({
        properties: [
            factory.PropertyAssignment.create({
                name: "httpMethod",
                initializer: factory.StringLiteral.create({ text: params.operationParams.httpMethod.toUpperCase() }),
            }),
            factory.ShorthandPropertyAssignment.create({
                name: methodType === "currying-function" ? "uri" : "url",
            }),
            factory.ShorthandPropertyAssignment.create({
                name: "headers",
            }),
            convertedParams.hasRequestBody &&
                factory.PropertyAssignment.create({
                    name: "requestBody",
                    initializer: Utils.generateVariableIdentifier(factory, "params.requestBody"),
                }),
            requestBodyEncoding &&
                factory.PropertyAssignment.create({
                    name: "requestBodyEncoding",
                    initializer: requestBodyEncoding,
                }),
            convertedParams.hasQueryParameters &&
                factory.PropertyAssignment.create({
                    name: "queryParameters",
                    initializer: factory.Identifier.create({ name: "queryParameters" }),
                }),
        ].flatMap(v => (v ? [v] : [])),
        multiLine: true,
    });
    const argumentsArray = [requestArgs, factory.Identifier.create({ name: "option" })];
    return factory.CallExpression.create({
        expression: expression,
        typeArguments: [],
        argumentsArray: argumentsArray,
    });
};
//# sourceMappingURL=CallRequest.js.map