import * as Api from "./api";
import type * as Types from "./types";
export interface Option {
    allowOperationIds?: string[];
    convertOption: Api.OpenApiTools.ConvertContext.Options;
}
export declare class CodeGenerator {
    private readonly entryPointOrDocument;
    private option?;
    private rootSchema;
    private resolvedReferenceDocument;
    private parser;
    constructor(entryPointOrDocument: string | Types.OpenApi.Document, option?: Option | undefined);
    private createParser;
    /**
     * Validate the OpenAPI Schema
     */
    validateOpenApiSchema(option?: Types.Validator.Option): void;
    /**
     * Provides TypeScript typedefs generated from OpenAPI Schema.
     *
     * @param generatorTemplate Template for when you want to change the code following a type definition
     * @returns String of generated code
     */
    generateTypeDefinition(generatorTemplates?: Types.CodeGenerator.CustomGenerator<any>[], allowOperationIds?: string[]): string;
    /**
     * Generate code using a template
     *
     * @param generatorTemplate
     * @returns String of generated code
     */
    generateCode(generatorTemplates: Types.CodeGenerator.CustomGenerator<any>[], allowOperationIds?: string[]): string;
    /**
     * Provides parameters extracted from OpenApi Schema
     */
    getCodeGeneratorParamsArray(allowOperationIds?: string[]): Types.CodeGenerator.Params[];
    /**
     * Provides types for parameters for Templates.FunctionalApiClient.
     *
     * This API will be moved to Templates in the future.
     */
    getAdditionalTypeDefinitionCustomCodeGenerator(): Types.CodeGenerator.CustomGenerator<undefined>;
}
