import type ts from "typescript";
import type * as OpenApi from "./OpenApi";
export type PickedParameter = Pick<OpenApi.Parameter, "name" | "in" | "required" | "style" | "explode">;
export interface OpenApiResponses {
    [statusCode: string]: OpenApi.Response;
}
export interface OpenApiOperation {
    httpMethod: string;
    requestUri: string;
    comment: string | undefined;
    deprecated: boolean;
    requestBody?: OpenApi.RequestBody;
    parameters?: OpenApi.Parameter[];
    responses: OpenApiResponses;
}
export interface ConvertedParams {
    escapedOperationId: string;
    argumentParamsTypeDeclaration: string;
    functionName: string;
    requestContentTypeName: string;
    responseContentTypeName: string;
    parameterName: string;
    pickedParameters: PickedParameter[];
    requestBodyName: string;
    requestContentTypes: string[];
    requestFirstContentType: string | undefined;
    has2OrMoreRequestContentTypes: boolean;
    responseErrorNames: string[];
    responseSuccessNames: string[];
    responseFirstSuccessName: string | undefined;
    has2OrMoreSuccessNames: boolean;
    successResponseContentTypes: string[];
    successResponseFirstContentType: string | undefined;
    has2OrMoreSuccessResponseContentTypes: boolean;
    hasAdditionalHeaders: boolean;
    hasQueryParameters: boolean;
    hasParameter: boolean;
    hasRequestBody: boolean;
}
export interface Params {
    operationId: string;
    convertedParams: ConvertedParams;
    operationParams: OpenApiOperation;
}
/**
 * Used to further transform the code created by the specified Generator Template.
 */
export type IntermediateCode = string | ts.Statement;
export type GenerateFunction<Option = {}> = (payload: Params[], option?: Option) => IntermediateCode[];
export interface OutputConfiguration {
    /**
     *
     */
    transform?: (params: IntermediateCode) => IntermediateCode[];
}
export interface CustomGenerator<T> {
    generator: GenerateFunction<T>;
    option?: T;
}
